import requests, pandas, time, random, numpy
from bs4 import BeautifulSoup, Comment
from teams import _tms


def _standardized(
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        tm_tag: tms.get(
            reference_team.lower(),
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _split_definition_and_values(
    tag,
    trow,
):

    splits_by_definitions = {
        'game_location': 'Location',
        'qtr': 'Qtr',
        'down': 'Down',
        'yds_split': 'YdsToGo',
        'down_distance': 'DownsAndYdsToGo',
        'field_position': 'FieldPosition',
        'pt_diff': 'ScoreDifferential',
        'situation': 'GameSituation',
        'snap_type': 'SnapTypeAndHuddle',
        'blitz_type': 'PassRushType',
    }

    splits_definition = splits_by_definitions.get(
        tag,
        'Unknown',
    )

    splits_type = trow.find(
        attrs={'data-stat': tag}
    ).text

    if splits_type == '10+':
        splits_type = '>10'

    splits_type = splits_type.replace(
        ' ',
        ''
    )
    splits_type = splits_type.replace(
        ',',
        ''
    )

    return splits_definition, splits_type


def _splits_stat(
    tag,
    trow,
):

    splits_stat = None

    if tag in [
        'plays',
        'rush_att',
        'rush_yds',
        'rush_td',
        'rush_first_down',
        'pass_cmp',
        'pass_att',
        'pass_yds',
        'pass_td',
        'pass_int',
        'pass_sacked',
        'pass_first_down',
    ]:

        try:
            splits_stats_text = trow.find(
                attrs={'data-stat': tag}
            ).text
        except AttributeError:
            return None

        if splits_stats_text:
            try:
                splits_stat = int(
                    splits_stats_text
                )
            except ValueError:
                splits_stat = None
        else:
            splits_stat = None
    
    else:

        try:
            splits_stats_text = trow.find(
                attrs={'data-stat': tag}
            ).text
        except AttributeError:
            return None

        if splits_stats_text:
            try:
                splits_stat = float(
                    splits_stats_text
                )
            except ValueError:
                splits_stat = None

            if tag == 'pass_cmp_perc':
                try:
                    splits_stat = round((splits_stat / 100), 3)
                except ZeroDivisionError:
                    splits_stat = 0
        else:
            splits_stat = None

    return splits_stat


def _season_splits(
    season,
    reference_alias,
    for_or_against
):

    dlist = []

    if for_or_against == 'For' or for_or_against == 'for':
        for_or_against = 'For'
        splits_stats_link = f'https://www.pro-football-reference.com/teams/{reference_alias}/{season}_splits.htm'
    elif for_or_against == 'Against' or for_or_against == 'against':
        for_or_against = 'Against'
        splits_stats_link = f'https://www.pro-football-reference.com/teams/{reference_alias}/{season}_opp_splits.htm'
    else:
        print('~'*100)
        print('Unable to Process Season Splits - Check Parameters')
        print('~'*100)
        return pandas.DataFrame()

    print('~'*100)
    print(f'Processing Splits Stats For: {splits_stats_link}')
    print('~'*100)

    try:
        res = requests.get(
            splits_stats_link
        )
    except requests.exceptions.RequestException:
        print('~'*100)
        print('Request Exception - Check Splits Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()

    standardized = _standardized(
        reference_alias
    )
    
    if res.status_code == 200:        
        
        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )    

        soup = BeautifulSoup(
            res.content, 
            'html.parser'
        )

        table = soup.find(
            'table',
            id='game_location_splits'
        )
        tbody = table.find(
            'tbody'
        )
        trows = tbody.find_all(
            'tr'
        )

        for trow in trows:

            split_statistics = {
                'season': season,
                **standardized,
            }
            
            splits_definition, splits_type = _split_definition_and_values(
                'game_location',
                trow,
            )

            split_statistics['splits_by'] = splits_definition
            split_statistics['splits_type'] = splits_type
            split_statistics['splits_side'] = for_or_against

            for display, tag in {
                'total_plays': 'plays',
                'yds_to_go': 'yds_to_go',
                'avg_yds_gained': 'yards',
                'rush_att': 'rush_att',
                'rush_yds': 'rush_yds',
                'rush_yds_per_att': 'rush_yds_per_att',
                'rush_tds': 'rush_td',
                'rush_first_downs': 'rush_first_down',
                'pass_cmp': 'pass_cmp',
                'pass_att': 'pass_att',
                'pass_cmp_pct': 'pass_cmp_perc',            
                'pass_yds': 'pass_yds',
                'pass_yds_per_att': 'pass_yds_per_att',
                'pass_adj_net_yds_per_att': 'pass_adj_net_yds_per_att',
                'pass_int': 'pass_int',
                'pass_tds': 'pass_td',
                'pass_first_downs': 'pass_first_down',
                'passer_rating': 'pass_rating',
                'times_sacked': 'pass_sacked',
            }.items():

                splits_stat = _splits_stat(
                    tag,
                    trow,
                )

                split_statistics[display] = splits_stat

            dlist.append(
                split_statistics
            )

        comments = soup.find_all(
            string = lambda text: isinstance(text, Comment)
        )
        
        for comment in comments:

            for split_type in [
                'id="qtr_splits"',
                'id="down_splits"',
                'id="yds_split_splits"',
                'id="down_distance_splits"',
                'id="field_position_splits"',
                'id="pt_diff_splits"',
                'id="situation_splits"',
                'id="snap_type_splits"',
                'id="blitz_type_splits"',
            ]:
            
                if split_type in comment.extract():

                    comment_soup = BeautifulSoup(
                        comment.extract(), 
                        'html.parser'
                    )
                    comment_table = comment_soup.find(
                        'table'
                    )
                    comment_tbody = comment_table.find(
                        'tbody'
                    )
                    comment_trows = comment_tbody.find_all(
                        'tr'
                    )

                    for comment_trow in comment_trows:    

                        split_statistics = {
                            'season': season,
                            **standardized,
                        }

                        splits_definitions_tags = {
                            'id="qtr_splits"' : 'qtr',
                            'id="down_splits"' : 'down',
                            'id="yds_split_splits"' : 'yds_split',
                            'id="down_distance_splits"' : 'down_distance',
                            'id="field_position_splits"' : 'field_position',
                            'id="pt_diff_splits"' : 'pt_diff',
                            'id="situation_splits"' : 'situation',
                            'id="snap_type_splits"' : 'snap_type',
                            'id="blitz_type_splits"' : 'blitz_type',
                        }

                        splits_definitions_tag = splits_definitions_tags.get(
                            split_type
                        )

                        splits_definition, splits_type = _split_definition_and_values(
                            splits_definitions_tag,
                            comment_trow,
                        )

                        split_statistics['splits_by'] = splits_definition
                        split_statistics['splits_type'] = splits_type
                        split_statistics['splits_side'] = for_or_against

                        for display, tag in {
                            'total_plays': 'plays',
                            'yds_to_go': 'yds_to_go',
                            'avg_yds_gained': 'yards',
                            'rush_att': 'rush_att',
                            'rush_yds': 'rush_yds',
                            'rush_yds_per_att': 'rush_yds_per_att',
                            'rush_tds': 'rush_td',
                            'rush_first_downs': 'rush_first_down',
                            'pass_cmp': 'pass_cmp',
                            'pass_att': 'pass_att',
                            'pass_cmp_pct': 'pass_cmp_perc',            
                            'pass_yds': 'pass_yds',
                            'pass_yds_per_att': 'pass_yds_per_att',
                            'pass_adj_net_yds_per_att': 'pass_adj_net_yds_per_att',
                            'pass_int': 'pass_int',
                            'pass_tds': 'pass_td',
                            'pass_first_downs': 'pass_first_down',
                            'passer_rating': 'pass_rating',
                            'times_sacked': 'pass_sacked',
                        }.items():

                            splits_stat = _splits_stat(
                                tag,
                                comment_trow,
                            )

                            split_statistics[display] = splits_stat

                        dlist.append(
                            split_statistics
                        )


    dset = pandas.DataFrame(
        dlist
    )

    dset = dset.replace({
        numpy.nan: None
    })

    dset['splits_stats_link'] = splits_stats_link

    return dset


from .season import _gamelogs
from .metadata import _gamelog_metadata
from .stats import _gamelog_statistics
from .scoring import _gamelog_scoring
from .roster import _gamelog_roster
from .snap_counts import _gamelog_snap_counts
from .splits import _season_splits
from .five_thirty_eight import _five_thirty_eight
from .teams import _tms, _splits_tms


def _tms():

    return {
        'arizona': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'arizona-cardinals': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'st-louis-cardinals': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'phoenix-cardinals': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'phoenix': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'cardinals': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'ari': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'crd': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'atlanta': {
            'market': 'Atlanta',
            'name': 'Falcons',
            'alias': 'ATL',
            'nano': 'mLspw4gJNTQGN',
        },
        'atlanta-falcons': {
            'market': 'Atlanta',
            'name': 'Falcons',
            'alias': 'ATL',
            'nano': 'mLspw4gJNTQGN',
        },
        'falcons': {
            'market': 'Atlanta',
            'name': 'Falcons',
            'alias': 'ATL',
            'nano': 'mLspw4gJNTQGN',
        },
        'atl': {
            'market': 'Atlanta',
            'name': 'Falcons',
            'alias': 'ATL',
            'nano': 'mLspw4gJNTQGN',
        },
        'baltimore': {
            'market': 'Baltimore',
            'name': 'Ravens',
            'alias': 'BAL',
            'nano': 'zyhFXj1nywrm1',
        },
        'baltimore-ravens': {
            'market': 'Baltimore',
            'name': 'Ravens',
            'alias': 'BAL',
            'nano': 'zyhFXj1nywrm1',
        },
        'ravens': {
            'market': 'Baltimore',
            'name': 'Ravens',
            'alias': 'BAL',
            'nano': 'zyhFXj1nywrm1',
        },
        'bal': {
            'market': 'Baltimore',
            'name': 'Ravens',
            'alias': 'BAL',
            'nano': 'zyhFXj1nywrm1',
        },
        'rav': {
            'market': 'Baltimore',
            'name': 'Ravens',
            'alias': 'BAL',
            'nano': 'zyhFXj1nywrm1',
        },
        'buffalo': {
            'market': 'Buffalo',
            'name': 'Bills',
            'alias': 'BUF',
            'nano': '3VqeLK0LS9rpw',
        },
        'buffalo-bills': {
            'market': 'Buffalo',
            'name': 'Bills',
            'alias': 'BUF',
            'nano': '3VqeLK0LS9rpw',
        },
        'bills': {
            'market': 'Buffalo',
            'name': 'Bills',
            'alias': 'BUF',
            'nano': '3VqeLK0LS9rpw',
        },
        'buf': {
            'market': 'Buffalo',
            'name': 'Bills',
            'alias': 'BUF',
            'nano': '3VqeLK0LS9rpw',
        },
        'carolina': {
            'market': 'Carolina',
            'name': 'Panthers',
            'alias': 'CAR',
            'nano': 'jJBGuksuDJF5L',
        },
        'carolina-panthers': {
            'market': 'Carolina',
            'name': 'Panthers',
            'alias': 'CAR',
            'nano': 'jJBGuksuDJF5L',
        },
        'panthers': {
            'market': 'Carolina',
            'name': 'Panthers',
            'alias': 'CAR',
            'nano': 'jJBGuksuDJF5L',
        },
        'car': {
            'market': 'Carolina',
            'name': 'Panthers',
            'alias': 'CAR',
            'nano': 'jJBGuksuDJF5L',
        },
        'chicago': {
            'market': 'Chicago',
            'name': 'Bears',
            'alias': 'CHI',
            'nano': '1vVmWGzUZ1rea',
        },
        'chicago-bears': {
            'market': 'Chicago',
            'name': 'Bears',
            'alias': 'CHI',
            'nano': '1vVmWGzUZ1rea',
        },
        'bears': {
            'market': 'Chicago',
            'name': 'Bears',
            'alias': 'CHI',
            'nano': '1vVmWGzUZ1rea',
        },
        'chi': {
            'market': 'Chicago',
            'name': 'Bears',
            'alias': 'CHI',
            'nano': '1vVmWGzUZ1rea',
        },
        'cincinnati': {
            'market': 'Cincinnati',
            'name': 'Bengals',
            'alias': 'CIN',
            'nano': '22ZdibiqvfA2D',
        },
        'cincinnati-bengals': {
            'market': 'Cincinnati',
            'name': 'Bengals',
            'alias': 'CIN',
            'nano': '22ZdibiqvfA2D',
        },
        'bengals': {
            'market': 'Cincinnati',
            'name': 'Bengals',
            'alias': 'CIN',
            'nano': '22ZdibiqvfA2D',
        },
        'cin': {
            'market': 'Cincinnati',
            'name': 'Bengals',
            'alias': 'CIN',
            'nano': '22ZdibiqvfA2D',
        },
        'cleveland': {
            'market': 'Cleveland',
            'name': 'Browns',
            'alias': 'CLE',
            'nano': 'IR4gIYmh269EI',
        },
        'cleveland-browns': {
            'market': 'Cleveland',
            'name': 'Browns',
            'alias': 'CLE',
            'nano': 'IR4gIYmh269EI',
        },
        'browns': {
            'market': 'Cleveland',
            'name': 'Browns',
            'alias': 'CLE',
            'nano': 'IR4gIYmh269EI',
        },
        'cle': {
            'market': 'Cleveland',
            'name': 'Browns',
            'alias': 'CLE',
            'nano': 'IR4gIYmh269EI',
        },
        'dallas': {
            'market': 'Dallas',
            'name': 'Cowboys',
            'alias': 'DAL',
            'nano': 'KdjU81i7YSFE7',
        },
        'dallas-cowboys': {
            'market': 'Dallas',
            'name': 'Cowboys',
            'alias': 'DAL',
            'nano': 'KdjU81i7YSFE7',
        },
        'cowboys': {
            'market': 'Dallas',
            'name': 'Cowboys',
            'alias': 'DAL',
            'nano': 'KdjU81i7YSFE7',
        },
        'dal': {
            'market': 'Dallas',
            'name': 'Cowboys',
            'alias': 'DAL',
            'nano': 'KdjU81i7YSFE7',
        },
        'denver': {
            'market': 'Denver',
            'name': 'Broncos',
            'alias': 'DEN',
            'nano': '6DclFuycdPaOB',
        },
        'denver-broncos': {
            'market': 'Denver',
            'name': 'Broncos',
            'alias': 'DEN',
            'nano': '6DclFuycdPaOB',
        },
        'broncos': {
            'market': 'Denver',
            'name': 'Broncos',
            'alias': 'DEN',
            'nano': '6DclFuycdPaOB',
        },
        'den': {
            'market': 'Denver',
            'name': 'Broncos',
            'alias': 'DEN',
            'nano': '6DclFuycdPaOB',
        },
        'detroit': {
            'market': 'Detroit',
            'name': 'Lions',
            'alias': 'DET',
            'nano': 'FCuH1wdksA2DQ',
        },
        'detroit-lions': {
            'market': 'Detroit',
            'name': 'Lions',
            'alias': 'DET',
            'nano': 'FCuH1wdksA2DQ',
        },
        'lions': {
            'market': 'Detroit',
            'name': 'Lions',
            'alias': 'DET',
            'nano': 'FCuH1wdksA2DQ',
        },
        'det': {
            'market': 'Detroit',
            'name': 'Lions',
            'alias': 'DET',
            'nano': 'FCuH1wdksA2DQ',
        },
        'green-bay': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },
        'green-bay-packers': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },
        'greenbay': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },
        'packers': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },
        'gb': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },
        'gnb': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },
        'houston': {
            'market': 'Houston',
            'name': 'Texans',
            'alias': 'HOU',
            'nano': 'UI0Ax4HUEXFh6',
        },
        'houston-texans': {
            'market': 'Houston',
            'name': 'Texans',
            'alias': 'HOU',
            'nano': 'UI0Ax4HUEXFh6',
        },
        'texans': {
            'market': 'Houston',
            'name': 'Texans',
            'alias': 'HOU',
            'nano': 'UI0Ax4HUEXFh6',
        },
        'hou': {
            'market': 'Houston',
            'name': 'Texans',
            'alias': 'HOU',
            'nano': 'UI0Ax4HUEXFh6',
        },
        'htx': {
            'market': 'Houston',
            'name': 'Texans',
            'alias': 'HOU',
            'nano': 'UI0Ax4HUEXFh6',
        },
        'indianapolis': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },
        'indianapolis-colts': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },
        'baltimore-colts': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },
        'colts': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },
        'ind': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },
        'clt': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },
        'jacksonville': {
            'market': 'Jacksonville',
            'name': 'Jaguars',
            'alias': 'IND',
            'nano': 'RtOjnciWkJPaL',
        },
        'jacksonville-jaguars': {
            'market': 'Jacksonville',
            'name': 'Jaguars',
            'alias': 'JAC',
            'nano': 'RtOjnciWkJPaL',
        },
        'jaguars': {
            'market': 'Jacksonville',
            'name': 'Jaguars',
            'alias': 'JAC',
            'nano': 'RtOjnciWkJPaL',
        },
        'jac': {
            'market': 'Jacksonville',
            'name': 'Jaguars',
            'alias': 'JAC',
            'nano': 'RtOjnciWkJPaL',
        },
        'jax': {
            'market': 'Jacksonville',
            'name': 'Jaguars',
            'alias': 'JAC',
            'nano': 'RtOjnciWkJPaL',
        },
        'kansas-city': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'kansas-city-chiefs': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'kansascity': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'chiefs': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'kc': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'kan': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'las-vegas': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'las-vegas-raiders': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'los-angeles-raiders': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'oakland-raiders': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'raiders': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'lasvegas': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'oakland': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'lvr': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'lv': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'oak': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'rai': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },
        'los-angeles-chargers': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },
        'san-diego-chargers': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },
        'chargers': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },
        'sdg': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },
        'sd': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },
        'lac': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },
        'los-angeles-rams': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'st-louis-rams': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'st-louis': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'rams': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'ram': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'stl': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'lar': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'la': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'miami': {
            'market': 'Miami',
            'name': 'Dolphins',
            'alias': 'MIA',
            'nano': 'mfbu6O5aMHbJg',
        },
        'miami-dolphins': {
            'market': 'Miami',
            'name': 'Dolphins',
            'alias': 'MIA',
            'nano': 'mfbu6O5aMHbJg',
        },
        'dolphins': {
            'market': 'Miami',
            'name': 'Dolphins',
            'alias': 'MIA',
            'nano': 'mfbu6O5aMHbJg',
        },
        'mia': {
            'market': 'Miami',
            'name': 'Dolphins',
            'alias': 'MIA',
            'nano': 'mfbu6O5aMHbJg',
        },
        'minnesota': {
            'market': 'Minnesota',
            'name': 'Vikings',
            'alias': 'MIN',
            'nano': 'AzOs5UWsXusSW',
        },
        'minnesota-vikings': {
            'market': 'Minnesota',
            'name': 'Vikings',
            'alias': 'MIN',
            'nano': 'AzOs5UWsXusSW',
        },
        'vikings': {
            'market': 'Minnesota',
            'name': 'Vikings',
            'alias': 'MIN',
            'nano': 'AzOs5UWsXusSW',
        },
        'min': {
            'market': 'Minnesota',
            'name': 'Vikings',
            'alias': 'MIN',
            'nano': 'AzOs5UWsXusSW',
        },
        'minn': {
            'market': 'Minnesota',
            'name': 'Vikings',
            'alias': 'MIN',
            'nano': 'AzOs5UWsXusSW',
        },
        'new-england': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'new-england-patriots': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'boston-patriots': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'patriots': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'ne': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'nwe': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'new-orleans': {
            'market': 'New Orleans',
            'name': 'Saints',
            'alias': 'NO',
            'nano': 'CI5mhfNa0Zqd2',
        },
        'new-orleans-saints': {
            'market': 'New Orleans',
            'name': 'Saints',
            'alias': 'NO',
            'nano': 'CI5mhfNa0Zqd2',
        },
        'saints': {
            'market': 'New Orleans',
            'name': 'Saints',
            'alias': 'NO',
            'nano': 'CI5mhfNa0Zqd2',
        },
        'no': {
            'market': 'New Orleans',
            'name': 'Saints',
            'alias': 'NO',
            'nano': 'CI5mhfNa0Zqd2',
        },
        'nor': {
            'market': 'New Orleans',
            'name': 'Saints',
            'alias': 'NO',
            'nano': 'CI5mhfNa0Zqd2',
        },
        'new-york-jets': {
            'market': 'New York',
            'name': 'Jets',
            'alias': 'NYJ',
            'nano': 'mexvMWO2v1hWn',
        },
        'jets': {
            'market': 'New York',
            'name': 'Jets',
            'alias': 'NYJ',
            'nano': 'mexvMWO2v1hWn',
        },
        'nyj': {
            'market': 'New York',
            'name': 'Jets',
            'alias': 'NYJ',
            'nano': 'mexvMWO2v1hWn',
        },
        'new-york-giants': {
            'market': 'New York',
            'name': 'Giants',
            'alias': 'NYG',
            'nano': 'n8CuHV9wbYCqF',
        },
        'giants': {
            'market': 'New York',
            'name': 'Giants',
            'alias': 'NYG',
            'nano': 'n8CuHV9wbYCqF',
        },
        'nyg': {
            'market': 'New York',
            'name': 'Giants',
            'alias': 'NYG',
            'nano': 'n8CuHV9wbYCqF',
        },
        'philadelphia': {
            'market': 'Philadelphia',
            'name': 'Eagles',
            'alias': 'PHI',
            'nano': 'hjjmgzxilGWdr',
        },
        'philadelphia-eagles': {
            'market': 'Philadelphia',
            'name': 'Eagles',
            'alias': 'PHI',
            'nano': 'hjjmgzxilGWdr',
        },
        'eagles': {
            'market': 'Philadelphia',
            'name': 'Eagles',
            'alias': 'PHI',
            'nano': 'hjjmgzxilGWdr',
        },
        'phi': {
            'market': 'Philadelphia',
            'name': 'Eagles',
            'alias': 'PHI',
            'nano': 'hjjmgzxilGWdr',
        },
        'pittsburgh': {
            'market': 'Pittsburgh',
            'name': 'Steelers',
            'alias': 'PIT',
            'nano': 'SlVgURDFI7EXX',
        },
        'pittsburgh-steelers': {
            'market': 'Pittsburgh',
            'name': 'Steelers',
            'alias': 'PIT',
            'nano': 'SlVgURDFI7EXX',
        },
        'steelers': {
            'market': 'Pittsburgh',
            'name': 'Steelers',
            'alias': 'PIT',
            'nano': 'SlVgURDFI7EXX',
        },
        'pit': {
            'market': 'Pittsburgh',
            'name': 'Steelers',
            'alias': 'PIT',
            'nano': 'SlVgURDFI7EXX',
        },
        'san-francisco': {
            'market': 'San Francisco',
            'name': '49ers',
            'alias': 'SF',
            'nano': 'IwSI92ZDKoazn',
        },
        'san-francisco-49ers': {
            'market': 'San Francisco',
            'name': '49ers',
            'alias': 'SF',
            'nano': 'IwSI92ZDKoazn',
        },
        '49ers': {
            'market': 'San Francisco',
            'name': '49ers',
            'alias': 'SF',
            'nano': 'IwSI92ZDKoazn',
        },
        'sf': {
            'market': 'San Francisco',
            'name': '49ers',
            'alias': 'SF',
            'nano': 'IwSI92ZDKoazn',
        },
        'sfo': {
            'market': 'San Francisco',
            'name': '49ers',
            'alias': 'SF',
            'nano': 'IwSI92ZDKoazn',
        },
        'seattle': {
            'market': 'Seattle',
            'name': 'Seahawks',
            'alias': 'SEA',
            'nano': '4M8AK5p0WXs1N',
        },
        'seattle-seahawks': {
            'market': 'Seattle',
            'name': 'Seahawks',
            'alias': 'SEA',
            'nano': '4M8AK5p0WXs1N',
        },
        'seahawks': {
            'market': 'Seattle',
            'name': 'Seahawks',
            'alias': 'SEA',
            'nano': '4M8AK5p0WXs1N',
        },
        'sea': {
            'market': 'Seattle',
            'name': 'Seahawks',
            'alias': 'SEA',
            'nano': '4M8AK5p0WXs1N',
        },
        'tampa-bay': {
            'market': 'Tampa Bay',
            'name': 'Buccaneers',
            'alias': 'TB',
            'nano': 'QMAZH1v922YLu',
        },
        'tampa-bay-buccaneers': {
            'market': 'Tampa Bay',
            'name': 'Buccaneers',
            'alias': 'TB',
            'nano': 'QMAZH1v922YLu',
        },
        'buccaneers': {
            'market': 'Tampa Bay',
            'name': 'Buccaneers',
            'alias': 'TB',
            'nano': 'QMAZH1v922YLu',
        },
        'tb': {
            'market': 'Tampa Bay',
            'name': 'Buccaneers',
            'alias': 'TB',
            'nano': 'QMAZH1v922YLu',
        },
        'tam': {
            'market': 'Tampa Bay',
            'name': 'Buccaneers',
            'alias': 'TB',
            'nano': 'QMAZH1v922YLu',
        },
        'tennessee': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'tennessee-titans': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'houston-oilers': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'tennessee-oilers': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'oilers': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'titans': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'ten': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'oti': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'washington': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'washington-football-team': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'washington-commanders': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'washington-redskins': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'football-team': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'football team': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'redskins': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'commanders': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'wft': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'was': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
        'wsh': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
    }


def _splits_tms():

    return {
        'crd': {
            'market': 'Arizona',
            'name': 'Cardinals',
            'alias': 'ARI',
            'nano': 'GJizlkzQFaqGA',
        },
        'atl': {
            'market': 'Atlanta',
            'name': 'Falcons',
            'alias': 'ATL',
            'nano': 'mLspw4gJNTQGN',
        },
        'rav': {
            'market': 'Baltimore',
            'name': 'Ravens',
            'alias': 'BAL',
            'nano': 'zyhFXj1nywrm1',
        },
        'buf': {
            'market': 'Buffalo',
            'name': 'Bills',
            'alias': 'BUF',
            'nano': '3VqeLK0LS9rpw',
        },
        'car': {
            'market': 'Carolina',
            'name': 'Panthers',
            'alias': 'CAR',
            'nano': 'jJBGuksuDJF5L',
        },
        'chi': {
            'market': 'Chicago',
            'name': 'Bears',
            'alias': 'CHI',
            'nano': '1vVmWGzUZ1rea',
        },
        'cin': {
            'market': 'Cincinnati',
            'name': 'Bengals',
            'alias': 'CIN',
            'nano': '22ZdibiqvfA2D',
        },
        'cle': {
            'market': 'Cleveland',
            'name': 'Browns',
            'alias': 'CLE',
            'nano': 'IR4gIYmh269EI',
        },
        'dal': {
            'market': 'Dallas',
            'name': 'Cowboys',
            'alias': 'DAL',
            'nano': 'KdjU81i7YSFE7',
        },
        'den': {
            'market': 'Denver',
            'name': 'Broncos',
            'alias': 'DEN',
            'nano': '6DclFuycdPaOB',
        },
        'det': {
            'market': 'Detroit',
            'name': 'Lions',
            'alias': 'DET',
            'nano': 'FCuH1wdksA2DQ',
        },
        'gnb': {
            'market': 'Green Bay',
            'name': 'Packers',
            'alias': 'GB',
            'nano': 'D5QY6bOxfn8ZL',
        },   
        'htx': {
            'market': 'Houston',
            'name': 'Texans',
            'alias': 'HOU',
            'nano': 'UI0Ax4HUEXFh6',
        },
        'clt': {
            'market': 'Indianapolis',
            'name': 'Colts',
            'alias': 'IND',
            'nano': 'LJmHetie5VbbO',
        },    
        'jax': {
            'market': 'Jacksonville',
            'name': 'Jaguars',
            'alias': 'JAC',
            'nano': 'RtOjnciWkJPaL',
        },
        'kan': {
            'market': 'Kansas City',
            'name': 'Chiefs',
            'alias': 'KC',
            'nano': 'oiGiwWPf0UqMC',
        },
        'rai': {
            'market': 'Las Vegas',
            'name': 'Raiders',
            'alias': 'LV',
            'nano': 'LZoDld8s63xUs',
        },     
        'sdg': {
            'market': 'Los Angeles',
            'name': 'Chargers',
            'alias': 'LAC',
            'nano': 'SjaMbkmeizJD4',
        },  
        'ram': {
            'market': 'Los Angeles',
            'name': 'Rams',
            'alias': 'LA',
            'nano': 'u98I1lDKQBwZ0',
        },
        'mia': {
            'market': 'Miami',
            'name': 'Dolphins',
            'alias': 'MIA',
            'nano': 'mfbu6O5aMHbJg',
        },
        'min': {
            'market': 'Minnesota',
            'name': 'Vikings',
            'alias': 'MIN',
            'nano': 'AzOs5UWsXusSW',
        },
        'nwe': {
            'market': 'New England',
            'name': 'Patriots',
            'alias': 'NE',
            'nano': 'VkxV91RtzHn65',
        },
        'nor': {
            'market': 'New Orleans',
            'name': 'Saints',
            'alias': 'NO',
            'nano': 'CI5mhfNa0Zqd2',
        },
        'nyj': {
            'market': 'New York',
            'name': 'Jets',
            'alias': 'NYJ',
            'nano': 'mexvMWO2v1hWn',
        },
        'nyg': {
            'market': 'New York',
            'name': 'Giants',
            'alias': 'NYG',
            'nano': 'n8CuHV9wbYCqF',
        },
        'phi': {
            'market': 'Philadelphia',
            'name': 'Eagles',
            'alias': 'PHI',
            'nano': 'hjjmgzxilGWdr',
        },
        'pit': {
            'market': 'Pittsburgh',
            'name': 'Steelers',
            'alias': 'PIT',
            'nano': 'SlVgURDFI7EXX',
        },
        'sfo': {
            'market': 'San Francisco',
            'name': '49ers',
            'alias': 'SF',
            'nano': 'IwSI92ZDKoazn',
        },
        'sea': {
            'market': 'Seattle',
            'name': 'Seahawks',
            'alias': 'SEA',
            'nano': '4M8AK5p0WXs1N',
        },
        'tam': {
            'market': 'Tampa Bay',
            'name': 'Buccaneers',
            'alias': 'TB',
            'nano': 'QMAZH1v922YLu',
        },
        'oti': {
            'market': 'Tennessee',
            'name': 'Titans',
            'alias': 'TEN',
            'nano': 'ltMyRE3yvVUaw',
        },
        'was': {
            'market': 'Washington',
            'name': 'Football Team',
            'alias': 'WAS',
            'nano': '9JtBk8kFeaGEs',
        },
    }


import requests, pandas, time, random
from bs4 import BeautifulSoup
from teams import _tms


def _standardized_with_side(
    tm_or_opp,
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        f'{tm_or_opp}_{tm_tag}': tms.get(
            reference_team.lower(),
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _is_playoff_filler(
    gamelog,
):
    event_date_text = gamelog.find(
        attrs={'data-stat': 'game_date'}
    ).text
    
    if event_date_text == 'Playoffs':
        return False
    else:
        return True


def _week(
    season,
    gamelog, 
):
    week_text = gamelog.find(
        attrs={
            'data-stat': 'week_num'
        }
    ).text

    week = None

    if 1970 <= season <= 1977:
        if week_text == "Division":
            week = 15
        elif week_text == "ConfChamp":
            week = 16
        elif week_text == "SuperBowl":
            week = 17
        else:
            week = int(week_text)  
    
    if 1978 <= season <= 1989:
        if week_text == "WildCard":
            week = 17
        elif week_text == "Division":
            week = 18
        elif week_text == "ConfChamp":
            week = 19
        elif week_text == "SuperBowl":
            week = 20
        else:
            week = int(week_text)  
    
    if 1990 <= season <= 2020:
        if week_text == "WildCard":
            week = 18
        elif week_text == "Division":
            week = 19
        elif week_text == "ConfChamp":
            week = 20
        elif week_text == "SuperBowl":
            week = 21
        else:
            week = int(week_text)

    if 2021 <= season:
        if week_text == "WildCard":
            week = 19
        elif week_text == "Division":
            week = 20
        elif week_text == "ConfChamp":
            week = 21
        elif week_text == "SuperBowl":
            week = 22
        else:
            week = int(week_text)

    return {
        'week': week
    }


def _week_day(
    gamelog,
):
    week_day = gamelog.find(
        attrs={'data-stat': 'game_day_of_week'}
    ).text
    
    return {
        'week_day': week_day
    }


def _event_date(
    gamelog,
):
    event_date = gamelog.find(
        attrs={'data-stat': 'game_date'}
    ).text

    return {
        'event_date': event_date
    }


def _alt_alias_and_market(
    tm_or_opp,
    side,
    gamelog,
):
    
    try:
        boxscore_href = gamelog.find(
            attrs={'data-stat': side}
        ).find('a').attrs['href']
        alias = boxscore_href.split('/')[2]
    except:
        alias = 'UNKNOWN'
    
    try:
        boxscore_text = gamelog.find(
            attrs={'data-stat': side}
        ).find('a').text
        market = boxscore_text.replace('.', '').replace('&', '').replace('(', '')
        market = market.replace(')', '').replace(' ', '-').lower().replace('*', '')
    except:
        market = 'UNKNOWN'

    return {
        f'{tm_or_opp}_alt_market': market,
        f'{tm_or_opp}_alt_alias': alias,
    }


def _locations(
    gamelog,
):

    location_text = gamelog.find(
        attrs={'data-stat': 'game_location'}
    ).text
    
    if location_text == '@':
        winner_location = 'A'
        loser_location = 'H'    
    elif location_text == '':
        winner_location = 'H'
        loser_location = 'A'    
    elif location_text == 'N':
        winner_location = 'N'
        loser_location = 'N'
    else:
        winner_location = 'N'
        loser_location = 'N'

    return {
        'tm_location': winner_location,
        'opp_location': loser_location,
    }


def _scores(
    gamelog,
):

    tm_score_text = gamelog.find(
        attrs={'data-stat': 'pts_win'}
    ).text
    
    opp_score_text = gamelog.find(
        attrs={'data-stat': 'pts_lose'}
    ).text
    
    try:
        tm_score = int(
            tm_score_text
        )
        opp_score = int(
            opp_score_text
        )
        return {
            'tm_score': tm_score,
            'opp_score': opp_score,
        }
    except ValueError:
        return {
            'tm_score': 0,
            'opp_score': 0,
        }


def _boxscore_stats_link(
    gamelog
):
    boxscore_stats_link_href = gamelog.find(
        attrs={'data-stat': 'boxscore_word'}
    ).find('a').attrs['href']
    
    boxscore_stats_link = f'https://www.pro-football-reference.com{boxscore_stats_link_href}'
    
    return {
        'boxscore_stats_link': boxscore_stats_link,
    }


def _gamelogs(
    season
):

    print('~'*100)
    print(f'Processing Season Gamelogs For: https://www.pro-football-reference.com/years/{season}/games.htm')
    print('~'*100)

    try:
        if season < 1970:
            print('~'*50)
            print('Gamelogs From 1970 to Present Only')
            print('~'*50)
            return pandas.DataFrame()
    except TypeError:
        print('~'*50)
        print('Season Should Be an Integer from 1970 to Present')
        print('~'*50)
        return pandas.DataFrame()

    try:
        res = requests.get(
            f'https://www.pro-football-reference.com/years/{season}/games.htm'
        )
    except requests.exceptions.RequestException:
        print('~'*100)
        print('Request Exception - Check Season Formatting')
        print('~'*100)
        return pandas.DataFrame()    

    dlist = []

    if res.status_code == 200:

        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )

        soup = BeautifulSoup(
            res.content, 
            'html.parser'
        )
        table = soup.find(
            'table',
            id='games'
        )
        tbody = table.find(
            'tbody'
        )
        table_rows = tbody.find_all(
            'tr'
        )

        for gamelog in table_rows:
            
            if not gamelog.attrs:

                if _is_playoff_filler(
                    gamelog
                ):
                    
                    week = _week(
                        season,
                        gamelog, 
                    )
                    week_day = _week_day(
                        gamelog
                    )
                    event_date = _event_date(
                        gamelog
                    )
                    tm_alt_alias_and_market = _alt_alias_and_market(
                        'tm', 
                        'winner', 
                        gamelog
                    )
                    opp_alt_alias_and_market = _alt_alias_and_market(
                        'opp', 
                        'loser', 
                        gamelog
                    )
                    tm_standardized = _standardized_with_side(
                        'tm',
                        tm_alt_alias_and_market.get('tm_alt_market'),
                    )
                    opp_standardized = _standardized_with_side(
                        'opp',
                        opp_alt_alias_and_market.get('opp_alt_market'),
                    )                    
                    locations = _locations(
                        gamelog
                    )
                    scores = _scores(
                        gamelog,
                    )
                    boxscore_stats_link = _boxscore_stats_link(
                        gamelog
                    )                 

                    if scores.get('tm_score') or scores.get('opp_score'):
                        status = 'closed'
                    else:
                        status = 'upcoming/postponed'

                    gamelog = {
                        'status': status,
                        'season': season,
                        **week,
                        **week_day,
                        **event_date,
                        **tm_standardized,
                        **tm_alt_alias_and_market,
                        **opp_standardized,
                        **opp_alt_alias_and_market,
                        **locations,
                        **scores,
                        **boxscore_stats_link,
                    }

                    dlist.append(
                        gamelog
                    )
        
    dset = pandas.DataFrame(
        dlist
    ).drop_duplicates()

    return dset


import requests, pandas, time, random, re
from bs4 import BeautifulSoup, Comment
from teams import _tms


def _season_and_event_date(
    boxscore_stats_link,
):

    date_regex =  re.findall(
        r'\d+',
        boxscore_stats_link
    )

    year = date_regex[0][0:4]
    month = date_regex[0][4:6]
    day = date_regex[0][6:8]

    if month == '01' or month == 1:
        season = int(year) - 1
        event_date = f'{year}-{month}-{day}'
    else:
        season = int(year)
        event_date = f'{year}-{month}-{day}'

    return {
        'season': season,
        'event_date': event_date,
    }


def _standardized(
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        tm_tag: tms.get(
            reference_team,
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _reference_team(
    caption,
):

    caption_text = caption.text
    reference_team = caption_text.replace(
        ' Snap Counts Table', ''
    ).lower()
    reference_team = reference_team.replace('.', '').replace('&', '')
    reference_team = reference_team.replace('(', '').replace(')', '')
    reference_team = reference_team.replace(' ', '-').lower().replace('*', '')

    return reference_team


def _player(
    trow,
):

    player = trow.find(
        attrs={'data-stat': 'player'}
    ).text
    
    return {
        'player': player
    }


def _player_reference(
    trow,
):

    try:
        player_href = trow.find(
            attrs={'data-stat': 'player'}
        ).find('a').attrs['href']
        return {
            'player_href': player_href
        }        
    except AttributeError:
        return {
            'player_href': None
        }


def _position(
    trow,
):

    position = trow.find(
        attrs={'data-stat': 'pos'}
    ).text
        
    return {
        'position': position
    }


def _snap_counts_stat(
    tag,
    trow,
):

    if tag in [
        'offense',
        'defense',
        'special_teams',
    ]:

        snap_count_text = trow.find(
            attrs={'data-stat': tag}
        ).text
        snap_count_stat = int(snap_count_text)

    else:
        
        snap_count_text = trow.find(
            attrs={'data-stat': tag}
        ).text
        snap_count_text = snap_count_text.replace('%', '')
        snap_count_stat = float(
            snap_count_text
        )
        snap_count_stat = round(
            (snap_count_stat / 100),
            3
        )

    return snap_count_stat


def _gamelog_snap_counts(
    boxscore_stats_link,
):

    print('~'*100)
    print(f'Processing Snap Counts For: {boxscore_stats_link}')
    print('~'*100)  
    
    try:
        season_and_event_date = _season_and_event_date(
            boxscore_stats_link,
        )
    except IndexError:
        print('~'*100)
        print('Unable to Process Gamelog Snap Counts - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()

    try:
        res = requests.get(
            boxscore_stats_link
        )
    except requests.exceptions.RequestException:
        print('~'*100)
        print('Request Exception - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()

    dlist = []
    
    if res.status_code == 200:        
        
        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )    

        soup = BeautifulSoup(
            res.content, 
            'html.parser'
        )
        comments = soup.find_all(
            string = lambda text: isinstance(text, Comment)
        )
        
        for comment in comments:

            for stat_side in ['id="vis_snap_counts"', 'id="home_snap_counts"']:
            
                if stat_side in comment.extract():

                    comment_soup = BeautifulSoup(
                        comment.extract(), 
                        'html.parser'
                    )
                    tbody = comment_soup.find(
                        'tbody'
                    )
                    trows = tbody.find_all(
                        'tr'
                    )
                    caption = comment_soup.find(
                        'caption'
                    )

                    reference_team = _reference_team(
                        caption
                    )
                    standardized = _standardized(
                        reference_team
                    )

                    for trow in trows:

                        player = _player(
                            trow
                        )
                        player_reference = _player_reference(
                            trow
                        )
                        position = _position(
                            trow
                        )

                        metadata = {
                            **season_and_event_date,
                            **standardized,
                            **player_reference,
                            **player,
                            **position,
                        }
                        stats = {
                            display: _snap_counts_stat(
                                tag, 
                                trow
                            )
                            for display, tag in {
                                'snap_count_offense': 'offense',
                                'snap_count_offense_pct': 'off_pct',
                                'snap_count_defense': 'defense',
                                'snap_count_defense_pct': 'def_pct',
                                'snap_count_special_teams': 'special_teams',
                                'snap_count_special_teams_pct': 'st_pct',
                            }.items()
                        }

                        snap_counts_stats = {
                            **metadata,
                            **stats,
                            'boxscore_stats_link': boxscore_stats_link,
                        }

                        dlist.append(
                            snap_counts_stats
                        )
                    
    dset = pandas.DataFrame(
        dlist
    )

    return dset


import requests, pandas, time, random, numpy, re
from bs4 import BeautifulSoup, Comment
from teams import _tms


def _season_and_event_date(
    boxscore_stats_link,
):

    date_regex =  re.findall(
        r'\d+',
        boxscore_stats_link
    )

    year = date_regex[0][0:4]
    month = date_regex[0][4:6]
    day = date_regex[0][6:8]

    if month == '01' or month == 1:
        season = int(year) - 1
        event_date = f'{year}-{month}-{day}'
    else:
        season = int(year)
        event_date = f'{year}-{month}-{day}'

    return {
        'season': season,
        'event_date': event_date,
    }


def _standardized(
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        tm_tag: tms.get(
            reference_team,
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _reference_team(
    stat_text,
    thead, 
):

    reference_team = thead.find(
        attrs={'data-stat': stat_text}
    ).text
    reference_team = reference_team.lower()
    
    return reference_team


def _rush_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[1]
        rush_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        rush_stats_text_split = rush_stats_text.replace('--', '-')
        rush_stats_text_split = rush_stats_text_split.split('-')
        rush_att = rush_stats_text_split[0]
        rush_att = int(rush_att)
        rush_yds = rush_stats_text_split[1]
        rush_yds = int(rush_yds)
        rush_tds = rush_stats_text_split[2]
        rush_tds = int(rush_tds)
        return {
            'rush_att': rush_att,
            'rush_yds': rush_yds,
            'rush_tds': rush_tds,
        }
    except ValueError:
        return {
            'rush_att': None,
            'rush_yds': None,
            'rush_tds': None,
        }
    

def _pass_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[2]
        pass_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        pass_stats_text_split = pass_stats_text.replace('--', '-')
        pass_stats_text_split = pass_stats_text_split.split('-')
        pass_cmp = pass_stats_text_split[0]
        pass_cmp = int(pass_cmp)
        pass_att = pass_stats_text_split[1]
        pass_att = int(pass_att)
        pass_yds = pass_stats_text_split[2]
        pass_yds = int(pass_yds)
        pass_tds = pass_stats_text_split[3]
        pass_tds = int(pass_tds)
        pass_int = pass_stats_text_split[4]
        pass_int = int(pass_int)

        try:
            pass_cmp_pct = round((pass_cmp / pass_att), 3)
        except ZeroDivisionError:
            pass_cmp_pct = 0

        return {
            'pass_cmp': pass_cmp,
            'pass_att': pass_att,
            'pass_cmp_pct': pass_cmp_pct,
            'pass_yds': pass_yds,
            'pass_tds': pass_tds,
            'pass_int': pass_int,
        }

    except ValueError:
        return {
            'pass_cmp': None,
            'pass_att': None,
            'pass_cmp_pct': None,
            'pass_yds': None,
            'pass_tds': None,
            'pass_int': None,
        }


def _passer_rating(
    pass_stats,
):

    pass_cmp = pass_stats.get(
        'pass_cmp'
    )
    pass_att = pass_stats.get(
        'pass_att'
    )
    pass_yds = pass_stats.get(
        'pass_yds'
    )
    pass_tds = pass_stats.get(
        'pass_tds'
    )
    pass_int = pass_stats.get(
        'pass_int'
    )

    blocks = {
        'a': (pass_cmp / pass_att - .3) * 5,
        'b': (pass_yds / pass_att - 3) * .25,
        'c': pass_tds / pass_att * 20,
        'd': 2.375 - pass_int / pass_att * 25,
    }
    blocks = {
        block: min((value, 2.375))
        for block, value in blocks.items()
    }
    blocks = {
        block: max((value, 0))
        for block, value in blocks.items()
    }
    passer_rating = sum(blocks.values()) / 6 * 100
    passer_rating = round(
        passer_rating,
        3,
    )

    return {
        'passer_rating': passer_rating
    }


def _sack_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[3]
        sack_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        sack_stats_text_split = sack_stats_text.split('-')
        times_sacked = sack_stats_text_split[0]
        times_sacked = int(times_sacked)
        yds_sacked_for = sack_stats_text_split[1]
        yds_sacked_for = int(yds_sacked_for)
        return {
            'times_sacked': times_sacked,
            'yds_sacked_for': yds_sacked_for,
        }
    except ValueError:
        return {
            'times_sacked': None,
            'yds_sacked_for': None,
        }


def _net_pass_yds(
    stat_text,
    tbody, 
):
    
    try:
        table_body_row = tbody[4]
        net_pass_yds = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        net_pass_yds = int(net_pass_yds)
        return {
            'net_pass_yds': net_pass_yds,
        }        
    except ValueError:
        return {
            'net_pass_yds': None,
        }


def _total_yds(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[5]
        total_yds = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        total_yds = int(total_yds)
        return {
            'total_yds': total_yds,
        }          
    except ValueError:
        return {
            'total_yds': None,
        }   


def _fumbles_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[6]
        sack_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        sack_stats_text_split = sack_stats_text.split('-')
        fumbles = sack_stats_text_split[0]
        fumbles = int(fumbles)
        fumbles_lost = sack_stats_text_split[1]
        fumbles_lost = int(fumbles_lost)
        return {
            'fumbles': fumbles,
            'fumbles_lost': fumbles_lost,
        }   
    except ValueError:
        return {
            'fumbles': None,
            'fumbles_lost': None,
        }   


def _turnover_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[7]
        turnovers = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        turnovers = int(turnovers)
        return {
            'turnovers': turnovers,
        }          
    except ValueError:
        return {
            'turnovers': None,
        }  


def _penalty_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[8]
        penalty_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        penalty_stats_text_split = penalty_stats_text.split('-')
        penalties = penalty_stats_text_split[0]
        penalties = int(penalties)
        penalty_yds = penalty_stats_text_split[1]
        penalty_yds = int(penalty_yds)
        return {
            'penalties': penalties,
            'penalty_yds': penalty_yds,
        }  
    except ValueError:
        return {
            'penalties': None,
            'penalty_yds': None,
        }  


def _first_downs_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[0]
        first_downs = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        first_downs = int(first_downs)
        return {
            'first_downs': first_downs,
        }
    except ValueError:
        return {
            'first_downs': None,
        }


def _third_down_stats(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[9]
        third_down_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        third_down_stats_text_split = third_down_stats_text.split('-')
        third_down_conv = third_down_stats_text_split[0]
        third_down_conv = int(third_down_conv)
        third_down_att = third_down_stats_text_split[1]
        third_down_att = int(third_down_att)
        
        try:
            third_down_conv_pct = round((third_down_conv / third_down_att), 3)
        except ZeroDivisionError:
            third_down_conv_pct = 0

        return {
            'third_down_conv': third_down_conv,
            'third_down_att': third_down_att,
            'third_down_conv_pct': third_down_conv_pct,
        }

    except ValueError:
        return {
            'third_down_conv': None,
            'third_down_att': None,
            'third_down_conv_pct': None,
        }


def _fourth_down_stats(
    stat_text,
    tbody, 
):
    
    try:
        table_body_row = tbody[10]
        fourth_down_stats_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        fourth_down_stats_text_split = fourth_down_stats_text.split('-')
        fourth_down_conv = fourth_down_stats_text_split[0]
        fourth_down_conv = int(fourth_down_conv)
        fourth_down_att = fourth_down_stats_text_split[1]
        fourth_down_att = int(fourth_down_att)
        
        try:
            fourth_down_conv_pct = round((fourth_down_conv / fourth_down_att), 3)
        except ZeroDivisionError:
            fourth_down_conv_pct = 0
        
        return {
            'fourth_down_conv': fourth_down_conv,
            'fourth_down_att': fourth_down_att,
            'fourth_down_conv_pct': fourth_down_conv_pct,
        }

    except ValueError:
        return {
            'fourth_down_conv': None,
            'fourth_down_att': None,
            'fourth_down_conv_pct': None,
        }


def _time_of_possession(
    stat_text,
    tbody, 
):

    try:
        table_body_row = tbody[11]
        time_of_possession_text = table_body_row.find(
            attrs={'data-stat': stat_text}
        ).text
        minutes, seconds = time_of_possession_text.split(':')
        minutes = int(minutes)
        seconds = int(seconds)
        time_of_possession = (minutes * 60) + seconds
        return {
            'time_of_possession': time_of_possession,
        }
    except ValueError:
        return {
            'time_of_possession': None,
        }


def _gamelog_statistics(
    boxscore_stats_link,
):

    print('~'*100)
    print(f'Processing Gamelog Statistics For: {boxscore_stats_link}')
    print('~'*100)

    try:
        season_and_event_date = _season_and_event_date(
            boxscore_stats_link,
        )
    except IndexError:
        print('~'*100)
        print('Unable to Process Gamelog Statistics - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()
    
    try:
        res = requests.get(
            boxscore_stats_link
        )
    except requests.exceptions.RequestException:
        print('~'*100)
        print('Request Exception - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()

    dlist = []

    if res.status_code == 200:        

        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )

        soup = BeautifulSoup(
            res.content, 
            'html.parser'
        )
        comments = soup.find_all(
            string = lambda text: isinstance(text, Comment)
        )
        
        for comment in comments:
            
            if 'id="team_stats"' in comment.extract():

                comment_soup = BeautifulSoup(
                    comment.extract(), 
                    'html.parser'
                )
                thead = comment_soup.find(
                    'thead'
                ).find(
                    'tr'
                )
                tbody = comment_soup.find(
                    'tbody'
                ).find_all(
                    'tr'
                )
                
                for stat_side in ['vis_stat', 'home_stat']:
                    
                    reference_team = _reference_team(
                        stat_side,
                        thead,
                    )
                    standardized = _standardized(
                        reference_team,
                    )
                    rush_stats = _rush_stats(
                        stat_side,
                        tbody,
                    )
                    pass_stats = _pass_stats(
                        stat_side,
                        tbody,
                    )
                    passer_rating = _passer_rating(
                        pass_stats,
                    )                    
                    net_pass_yds = _net_pass_yds(
                        stat_side,
                        tbody,
                    )
                    total_yds = _total_yds(
                        stat_side,
                        tbody,
                    )
                    sack_stats = _sack_stats(
                        stat_side,
                        tbody,
                    )
                    fumbles_stats = _fumbles_stats(
                        stat_side,
                        tbody,
                    )
                    turnover_stats = _turnover_stats(
                        stat_side,
                        tbody,
                    )
                    penalty_stats = _penalty_stats(
                        stat_side,
                        tbody,
                    )
                    first_downs_stats = _first_downs_stats(
                        stat_side,
                        tbody,
                    )                    
                    third_down_stats = _third_down_stats(
                        stat_side,
                        tbody,
                    )
                    fourth_down_stats = _fourth_down_stats(
                        stat_side,
                        tbody,
                    )
                    time_of_possession = _time_of_possession(
                        stat_side,
                        tbody,
                    )

                    statistics = {
                        **season_and_event_date,
                        **standardized,
                        **rush_stats,
                        **pass_stats,
                        **passer_rating,
                        **net_pass_yds,
                        **total_yds,
                        **sack_stats,
                        **fumbles_stats,
                        **turnover_stats,
                        **penalty_stats,
                        **first_downs_stats,
                        **third_down_stats,
                        **fourth_down_stats,
                        **time_of_possession,
                        'boxscore_stats_link': boxscore_stats_link,
                    }

                    dlist.append(
                        statistics
                    )

    dset = pandas.DataFrame(
        dlist
    )
    
    dset = dset.replace({
        numpy.nan: None
    })

    return dset


import requests, pandas, time, random, re
from bs4 import BeautifulSoup
from teams import _tms


def _season_and_event_date(
    boxscore_stats_link,
):

    date_regex =  re.findall(
        r'\d+',
        boxscore_stats_link
    )

    year = date_regex[0][0:4]
    month = date_regex[0][4:6]
    day = date_regex[0][6:8]

    if month == '01' or month == 1:
        season = int(year) - 1
        event_date = f'{year}-{month}-{day}'
    else:
        season = int(year)
        event_date = f'{year}-{month}-{day}'

    return {
        'season': season,
        'event_date': event_date,
    }


def _standardized(
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        tm_tag: tms.get(
            reference_team.lower(),
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _standardized_with_side(
    tm_or_opp,
    reference_team,
):

    tms = _tms()    

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        f'{tm_or_opp}_{tm_tag}': tms.get(
            reference_team.lower(),
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized
    

def _srf_home_vis_tms(
    thead,
):

    home_tm = thead.find(
        attrs={'data-stat': 'home_team_score'}
    ).text

    vis_tm = thead.find(
        attrs={'data-stat': 'vis_team_score'}
    ).text

    return home_tm, vis_tm


def _time(
    trow,
):

    trow_text = trow.find(
        attrs={'data-stat': 'time'}
    ).text
    
    try:
        game_minutes, game_seconds = trow_text.split(':')
        game_seconds_breakdown = int(game_minutes) * 60
        time = game_seconds_breakdown + int(game_seconds)
        
        return {
            'time': time
        }
    except ValueError:
        return {
            'time': None
        }


def _scoring_team(
    trow,
):

    scoring_team = trow.find(
        attrs={'data-stat': 'team'}
    ).text
    
    scoring_standardized = _standardized(
        scoring_team
    )

    scoring_team = scoring_standardized.get(
        'name'
    )

    return {
        'scoring_team': scoring_team,
    }


def _quarter(
    trow,
):

    quarter = trow.find(
        attrs={'data-stat': 'quarter'}
    ).text

    if quarter == 'OT' or quarter == 'ot':
        quarter = 5

    try:
        quarter = int(quarter)
        return {
            'quarter': quarter
        }
    except ValueError:
        return {
            'quarter': None
        }


def _description(
    trow,
):

    description = trow.find(
        attrs={'data-stat': 'description'}
    ).text
    
    return {
        'description': description
    }


def _scores(
    trow,
):

    tm_score = trow.find(
        attrs={'data-stat': 'home_team_score'}
    ).text

    opp_score = trow.find(
        attrs={'data-stat': 'vis_team_score'}
    ).text
    
    try:
        tm_score = int(
            tm_score
        )
        opp_score = int(
            opp_score
        )
        return {
            'tm_score': tm_score,
            'opp_score': opp_score,
        }
    except ValueError:
        return {
            'tm_score': 0,
            'opp_score': 0,
        }


def _gamelog_scoring(
    boxscore_stats_link,
):

    print('~'*100)
    print(f'Processing Game Scoring For: {boxscore_stats_link}')
    print('~'*100)
    
    try:
        season_and_event_date = _season_and_event_date(
            boxscore_stats_link,
        )
    except IndexError:
        print('~'*100)
        print('Unable to Process Gamelog Snap Counts - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()

    try:
        res = requests.get(
            boxscore_stats_link
        )
    except requests.exceptions.RequestException:
        print('~'*100)
        print('Request Exception - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()
    
    dlist = []

    if res.status_code == 200:        
        
        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )   

        soup = BeautifulSoup(
            res.content, 
            'html.parser'
        )
        table = soup.find(
            'table',
            id='scoring'
        )
        tbody = table.find(
            'tbody'
        )
        thead = table.find(
            'thead'
        )        
        trows = tbody.find_all(
            'tr'
        )
        reference_home_team, reference_vis_team = _srf_home_vis_tms(
            thead
        )
        tm_standardized = _standardized_with_side(
            'tm',
            reference_home_team,
        )
        opp_standardized = _standardized_with_side(
            'opp',
            reference_vis_team,
        )

        for trow in trows:

            quarter = _quarter(
                trow
            )
            time_of_score = _time(
                trow
            )
            scoring_team = _scoring_team(
                trow
            )
            description = _description(
                trow
            )

            scores = _scores(
                trow,
            )

            scoring_data = {
                **season_and_event_date,
                **tm_standardized,
                **opp_standardized,
                **quarter,
                **time_of_score,
                **scoring_team,
                **scores,
                **description,
                'boxscore_stats_link': boxscore_stats_link,
            }

            dlist.append(
                scoring_data
            )
                    
    dset = pandas.DataFrame(
        dlist
    )
    
    dset['quarter'].ffill(inplace=True)

    return dset


import requests, pandas, time, random, re
from bs4 import BeautifulSoup, Comment
from teams import _tms


def _season_and_event_date(
    boxscore_stats_link,
):

    date_regex =  re.findall(
        r'\d+',
        boxscore_stats_link
    )

    year = date_regex[0][0:4]
    month = date_regex[0][4:6]
    day = date_regex[0][6:8]

    if month == '01' or month == 1:
        season = int(year) - 1
        event_date = f'{year}-{month}-{day}'
    else:
        season = int(year)
        event_date = f'{year}-{month}-{day}'

    return {
        'season': season,
        'event_date': event_date,
    }


def _standardized(
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        tm_tag: tms.get(
            reference_team.lower(),
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _standardized_with_side(
    tm_or_opp,
    reference_team,
):

    tms = _tms()

    tm_tags = (
        'nano',
        'market',
        'name',
        'alias',
    )

    standardized = {
        f'{tm_or_opp}_{tm_tag}': tms.get(
            reference_team.lower(),
        ).get(
            tm_tag
        )
        for tm_tag in tm_tags
    }

    return standardized


def _reference_teams(
    thead,
):

    home_tm = thead.find(
        attrs={'data-stat': 'home_team_score'}
    ).text

    vis_tm = thead.find(
        attrs={'data-stat': 'vis_team_score'}
    ).text

    return home_tm, vis_tm


def _trow_stat_info(
    trow,
):

    try:
        trow_stat_info = trow.find(
            attrs={"data-stat": "info"}
        ).text
    except:
        trow_stat_info = None
    
    return trow_stat_info


def _toss_data(
    trow,
):

    won_toss_text = trow.find(
        attrs={'data-stat': 'stat'}
    ).text
    won_toss_text = won_toss_text.replace(
        ' (deferred)'
        ,''
    ).lower()
    
    standardized = _standardized(
        won_toss_text
    )
    alias = standardized.get(
        'alias'
    )
    
    return alias


def _toss_decision(
    season,
    trow,
):

    decision = None

    won_toss_text = trow.find(
        attrs={'data-stat': 'stat'}
    ).text

    if won_toss_text:
        if season >= 2009:
            if 'deferred' in won_toss_text:
                decision = 'Deferred'
            else:
                decision = 'Accepted'
        else:
            decision = None
        
    return decision


def _roof_type(
    trow,
):

    roof_type = trow.find(
        attrs={'data-stat': 'stat'}
    ).text

    if roof_type:
        roof_type = roof_type.capitalize()

    return {
        'roof_type': roof_type
    }


def _surface_type(
    trow,
):

    surface_type = trow.find(
        attrs={'data-stat': 'stat'}
    ).text

    if surface_type:
        surface_type = surface_type.capitalize()

    return {
        'surface_type': surface_type
    }


def _duration(
    trow,
):

    duration_text = trow.find(
        attrs={'data-stat': 'stat'}
    ).text
    
    try:
        game_hours, game_minutes = duration_text.split(':')
        game_hours_in_minutes = int(game_hours) * 60
        duration = game_hours_in_minutes + int(game_minutes)
        return {
            'duration': duration
        }
    except ValueError:
        return {
            'duration': None
        }


def _attendance(
    trow,
):

    attendance = trow.find(
        attrs={'data-stat': 'stat'}
    ).text.replace(',', '')
    try:
        attendance = int(
            attendance
        )
        return {
            'attendance': attendance
        }        
    except ValueError:
        return {
            'attendance': None
        }


def _weather(
    trow,
):
    
    temperature, humidity_pct, wind_speed = None, None, None
    
    weather_text = trow.find(
        attrs={'data-stat': 'stat'}
    ).text
    
    weather_list = weather_text.split(',')

    for weather_text in weather_list:

        if 'degree' in weather_text:
            temperature = re.findall(
                r'\d+', 
                weather_text
            )
            temperature = float(temperature[0])

        if 'humidity' in weather_text:
            humidity_pct = re.findall(
                r'\d+', 
                weather_text
            )
            humidity_pct = float(humidity_pct[0])
 
        if 'mph' in weather_text:
            wind_speed = re.findall(
                r'\d+', 
                weather_text
            )
            wind_speed = float(wind_speed[0])

        if 'no wind' in weather_text:
            wind_speed = 0.0
    
    return {
        'temperature': temperature,
        'humidity_pct': humidity_pct,
        'wind_speed': wind_speed,
    }


def _vegas_spread_market(
    tm_text,
):
    vegas_spread_market = tm_text.replace('.', '').replace('&', '')
    vegas_spread_market = vegas_spread_market.replace('(', '').replace(')', '')
    vegas_spread_market = vegas_spread_market.replace(' ', '-').lower().replace('*', '')
    
    return vegas_spread_market


def _spreads(
    tm_standardized, 
    opp_standardized, 
    vegas_spread_and_total_text,
):

    vegas_spread_text = vegas_spread_and_total_text.get(
        'vegas_spread_text'
    )

    vegas_spread_text

    if vegas_spread_text == 'Pick' or vegas_spread_text == 'pick':
        tm_spread = 0.0
        opp_spread = 0.0
        return tm_spread, opp_spread

    tm_text = vegas_spread_text.split('-')
    tm_text = tm_text[0]
    tm_text = tm_text.strip()

    reference_team = _vegas_spread_market(
        tm_text
    )

    standardized = _standardized(
        reference_team
    )

    vegas_line_text = vegas_spread_text.split('-')
    vegas_line_text = vegas_line_text[-1]
    vegas_line = float(vegas_line_text) * -1

    if standardized.get('market') == tm_standardized.get('tm_market'):
        tm_spread = vegas_line
        opp_spread = vegas_line * -1

    if standardized.get('market') == opp_standardized.get('opp_market'):
        tm_spread = vegas_line * -1
        opp_spread = vegas_line

    return {
        'tm_spread': tm_spread,
        'opp_spread': opp_spread,
    }


def _total(
    vegas_spread_and_total_text,
):

    vegas_total_text = vegas_spread_and_total_text.get(
        'vegas_total_text'
    )

    try:
        total = re.findall(
            r"[-+]?(?:\d*\.\d+|\d+)", 
            vegas_total_text
        )
        total = float(total[0])
    except TypeError:
        total = None
        
    return {
        'total': total
    }


def _vegas_spread_and_total_text(
    comments,
):

    vegas_spread_text, vegas_total_text = None, None

    for comment in comments:

        if 'Vegas Line' in comment.extract():

            comment_soup = BeautifulSoup(
                comment.extract(), 
                'html.parser'
            )
            table_trs = comment_soup.find(
                'table', 
                id='game_info'
            ).find_all('tr')

            for tr in table_trs:
                try:
                    if tr.find('th').text == 'Vegas Line':
                        vegas_spread = tr.find('td')
                        vegas_spread_text = vegas_spread.text
                    if tr.find('th').text == 'Over/Under':
                        vegas_total = tr.find('td')
                        vegas_total_text = vegas_total.text
                except:
                    next

    return {
        'vegas_spread_text': vegas_spread_text,
        'vegas_total_text': vegas_total_text,
    }
    

def _gamelog_metadata(
    boxscore_stats_link, 
):
    
    try:
        season_and_event_date = _season_and_event_date(
            boxscore_stats_link,
        )
    except IndexError:
        print('~'*100)
        print('Unable to Process Gamelog Metadata - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()
    
    try:
        res = requests.get(
            boxscore_stats_link
        )
    except requests.exceptions.RequestException:
        print('~'*100)
        print('Request Exception - Check Boxscore Stats Link Formatting')
        print('~'*100)
        return pandas.DataFrame()

    print('~'*100)
    print(f'Processing Gamelog Metadata For: {boxscore_stats_link}')
    print('~'*100)

    meta = {
        'tm_spread': None,
        'opp_spread': None,
        'total': None,
        'attendance': None,
        'duration': None,
        'roof_type': None,
        'surface_type': None,
        'won_toss': None,
        'won_toss_decision': None,
        'won_toss_overtime': None,
        'won_toss_overtime_decision': None,
        'temperature': None,
        'humidity_pct': None,
        'wind_speed': None,
        'boxscore_stats_link': boxscore_stats_link,
    }

    res = requests.get(
        boxscore_stats_link
    )
    
    if res.status_code == 200:        
        
        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )
        soup = BeautifulSoup(
            res.content, 
            'html.parser'
        )
        comments = soup.find_all(
            string = lambda text: isinstance(text, Comment)
        )
        table = soup.find(
            'table',
            id='scoring'
        )
        thead = table.find(
            'thead'
        )
        reference_home_team, reference_vis_team = _reference_teams(
            thead
        )
        tm_standardized = _standardized_with_side(
            'tm',
            reference_home_team,
        )
        opp_standardized = _standardized_with_side(
            'opp',
            reference_vis_team,
        )
        vegas_spread_and_total_text = _vegas_spread_and_total_text(
            comments,
        )        
        spreads = _spreads(
            tm_standardized, 
            opp_standardized, 
            vegas_spread_and_total_text
        )
        total = _total(
            vegas_spread_and_total_text
        )
        meta = {
            **meta,
            **spreads,
            **total,
        }

        for comment in comments:

            if 'id="game_info"' in comment.extract():

                comment_soup = BeautifulSoup(
                    comment.extract(), 
                    'html.parser'
                )
                table = comment_soup.find(
                    'table'
                )
                trows = table.find_all(
                    'tr'
                )     

                for trow in trows:
                    
                    trow_stat_info = _trow_stat_info(
                        trow
                    )
                    
                    if trow_stat_info:
                        
                        if 'Won Toss' in trow_stat_info:
                            won_toss = _toss_data(
                                trow,
                            )
                            won_toss_decision = _toss_decision(
                                season_and_event_date.get(
                                    'season'
                                ),
                                trow,
                            )
                            meta['won_toss'] = won_toss
                            meta['won_toss_decision'] = won_toss_decision
                        if 'Won OT Toss' in trow_stat_info:
                            won_toss_overtime = _toss_data(
                                trow
                            )
                            won_toss_overtime_decision = _toss_decision(
                                season_and_event_date.get(
                                    'season'
                                ),
                                trow,
                            )
                            meta['won_toss_overtime'] = won_toss_overtime
                            meta['won_toss_overtime_decision'] = won_toss_overtime_decision
                        if 'Roof' in trow_stat_info:
                            roof_type = _roof_type(
                                trow
                            )
                            meta = {
                                **meta,
                                **roof_type,
                            }
                        if 'Surface' in trow_stat_info:
                            surface_type = _surface_type(
                                trow
                            )
                            meta = {
                                **meta,
                                **surface_type,
                            }
                        if 'Duration' in trow_stat_info:
                            duration = _duration(
                                trow
                            )
                            meta = {
                                **meta,
                                **duration,
                            }
                        if 'Attendance' in trow_stat_info:
                            attendance = _attendance(
                                trow
                            )
                            meta = {
                                **meta,
                                **attendance,
                            }
                        if 'Weather' in trow_stat_info:
                            weather = _weather(
                                trow
                            )
                            meta = {
                                **meta,
                                **weather,
                            }

    metadata = {
        **season_and_event_date,
        **tm_standardized,
        **opp_standardized,
        **meta,
    }

    dset = pandas.DataFrame([
        metadata
    ])

    return dset


import pandas, numpy, requests, io, random, time
from teams import _tms


def _standardized_by_tag(
    tag,
    reference_team,
):

    tms = _tms()

    by_tag = tms.get(
        reference_team.lower(), {}
    ).get(
        tag
    )

    return by_tag


def _five_thirty_eight():

    print('~'*100)
    print(f'Processing fiveThirtyEight Elo Dataset')
    print('~'*100)

    res = requests.get(
        'https://projects.fivethirtyeight.com/nfl-api/nfl_elo.csv', 
        stream=True
    )

    if res.status_code == 200:

        sleeptime = random.uniform(
            2.5, 
            3.5,
        )
        time.sleep(
            sleeptime
        )  

        dset = pandas.read_csv(
            io.BytesIO(res.content)
        )

        dset = dset[
            dset['season'] >= 1970
        ].reset_index(
            drop=True
        )

        dset = dset.replace({
            numpy.nan: None
        })

        for tag in (
            'nano',
            'market',
            'name',
            'alias',
        ):
            for side in (
                'team1',
                'team2',
            ):
                if side == 'team1':
                    tm_or_opp = 'tm'
                else:
                    tm_or_opp = 'opp'

                dset[f'{tm_or_opp}_{tag}'] = dset[side].apply(
                    lambda x: _standardized_by_tag(
                        tag,
                        x,
                    )
                )

        dset = dset[[
            'date',
            'season',
            'neutral',
            'playoff',
            'tm_nano',
            'tm_market',
            'tm_name',
            'tm_alias',
            'team1',
            'opp_nano',
            'opp_market',
            'opp_name',
            'opp_alias',
            'team2',
            'score1',
            'score2',
            'elo1_pre',
            'elo2_pre',
            'elo_prob1',
            'elo_prob2',
            'elo1_post',
            'elo2_post',
            'qbelo1_pre',
            'qbelo2_pre',
            'qb1',
            'qb2',
            'qb1_value_pre',
            'qb2_value_pre',
            'qb1_adj',
            'qb2_adj',
            'qbelo_prob1',
            'qbelo_prob2',
            'qb1_game_value',
            'qb2_game_value',
            'qb1_value_post',
            'qb2_value_post',
            'qbelo1_post',
            'qbelo2_post',
            'quality',
            'importance',
            'total_rating',
        ]]

        dset = dset.rename(
            columns={
                'date': 'event_date',
                'team1': 'tm_alt_alias',
                'team2': 'opp_alt_alias',
                'score1': 'tm_score',
                'score2': 'opp_score',
                'elo1_pre': 'tm_elo_pre',
                'elo2_pre': 'opp_elo_pre',
                'elo_prob1': 'tm_elo_win_prob',
                'elo_prob2': 'opp_elo_win_prob',
                'elo1_post': 'tm_elo_post',
                'elo2_post': 'opp_elo_post',
                'qbelo1_pre': 'tm_qb_elo_pre',
                'qbelo2_pre': 'opp_qb_elo_pre',
                'qb1': 'tm_qb',
                'qb2': 'opp_qb',
                'qb1_value_pre': 'tm_qb_elo_value_pre',
                'qb2_value_pre': 'opp_qb_elo_value_pre',
                'qb1_adj': 'tm_qb_elo_adj',
                'qb2_adj': 'opp_qb_elo_adj',
                'qbelo_prob1': 'tm_qb_elo_win_prob',
                'qbelo_prob2': 'opp_qb_elo_win_prob',
                'qb1_game_value': 'tm_qb_game_value',
                'qb2_game_value': 'opp_qb_game_value',
                'qb1_value_post': 'tm_qb_post_game_value',
                'qb2_value_post': 'opp_qb_post_game_value',
                'qbelo1_post': 'tm_qb_elo_post_game',
                'qbelo2_post': 'opp_qb_elo_post_game',
            }
        )

        return dset


from setuptools import setup, find_packages

VERSION = '0.1.0' 
DESCRIPTION = "Datasets and Scraping Functions for NFL Data"
LONG_DESCRIPTION = """
This package was inspired by the creators of nflscrapR and nflfastR and 
the tremendous influence they have had on the open-source NFL community. 
The functionality of nflscraPy was designed to allow Python users to easily ingest boxscore and 
seasonal data from publicly available resourses, in particular, Pro Football Reference.
Hopefully this package builds upon the availabilty of open-source resources for the football and data analytics community.
"""

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="nflscraPy", 
        version=VERSION,
        license="MIT License",
        author="Tyler Durden",
        author_email="blankpagelabs@gmail.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=[
            "beautifulsoup4>=4.11.1",
            "bs4>=0.0.1",
            "certifi>=2022.12.7",
            "charset-normalizer>=2.1.1",
            "idna>=3.4",
            "numpy>=1.24.1",
            "pandas>=1.5.2",
            "python-dateutil>=2.8.2",
            "pytz>=2022.7",
            "requests>=2.28.1",
            "six>=1.16.0",
            "soupsieve>=2.3.2.post1",
            "urllib3>=1.26.13",
        ],
        keywords=[
            'python', 
            'scraper',
            'NFL Data',
            'Pro Football Reference',
            'Sports Reference',
            'fiveThirtyEight',
        ],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Programming Language :: Python :: 3",
            "Operating System :: OS Independent"
        ]
)